"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Collection = void 0;
const mongodb_1 = require("./mongodb");
const errors_1 = require("./errors");
class Collection extends mongodb_1.MongoCollection {
    constructor(db, name, options) {
        super(db, name, options);
    }
    countDocuments(filter, options) {
        return super.countDocuments(filter, options);
    }
    upsert(filter, update, options) {
        return super.updateOne(filter, update, {
            upsert: true,
            ...options
        });
    }
    withDocumentLock(options) {
        const { filter, withLockedDocument } = options;
        return this.lock(options)
            .then(document => Promise.resolve(withLockedDocument(document)))
            .then(result => this.unlockWithResult(filter, result))
            .catch(async (error) => {
            if (!(error instanceof errors_1.DocumentPinnedError)) {
                await this.unlock(filter);
            }
            throw error;
        });
    }
    async lock(options) {
        const { filter, document } = options;
        const lock = {
            _lockPinned: true,
            _lockOwnerHostname: process.env.HOSTNAME || 'localhost'
        };
        const update = {
            $set: lock,
            $setOnInsert: document
        };
        const filters = {
            ...filter,
            _lockPinned: { $exists: false }
        };
        return super.findOneAndUpdate(filters, update, { upsert: true, returnDocument: 'after' })
            .then(result => result)
            .catch(error => {
            if (error.code === 11000) {
                throw new errors_1.DocumentPinnedError('Document failed to insert a pin due to index constraint');
            }
            throw error;
        });
    }
    unlock(filter) {
        const lock = {
            _lockPinned: true,
            _lockOwnerHostname: true
        };
        return super.updateOne(filter, {
            $unset: lock
        });
    }
    unlockWithResult(filters, result) {
        return this.unlock(filters).then(() => result);
    }
}
exports.Collection = Collection;
//# sourceMappingURL=collection.js.map