"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CollectionMetadata = void 0;
const common_1 = require("@nestjs/common");
const constants_1 = require("./constants");
class CollectionMetadata {
    entity;
    static registry = new Map();
    logger = new common_1.Logger(CollectionMetadata.name);
    constructor(entity) {
        this.entity = entity;
    }
    async synchronize(collection) {
        const entityMetadata = Reflect.getMetadata(constants_1.ENTITY_METADATA, this.entity.prototype) || {
            columns: []
        };
        const groupIndex = entityMetadata.columns.reduce((index, column) => {
            if (column.options.unique) {
                return {
                    ...index,
                    unique: true,
                    keys: [...index.keys, column.propertyKey]
                };
            }
            if (column.options.sparse) {
                return {
                    ...index,
                    sparse: true,
                    keys: [...index.keys, column.propertyKey]
                };
            }
            return index;
        }, { keys: [], unique: false, sparse: false });
        if (groupIndex.keys.length) {
            const uniqueIndexSpecification = groupIndex.keys.reduce((spec, key) => ({ ...spec, [key]: 1 }), {});
            await collection.createIndex(uniqueIndexSpecification, {
                name: 'uniqueDocumentIndex',
                unique: groupIndex.unique,
                sparse: groupIndex.sparse
            })
                .catch(error => this.logger.debug(error.message, error.stack));
        }
    }
    static collect(entity) {
        if (this.registry.has(entity)) {
            return this.registry.get(entity);
        }
        const metadata = new CollectionMetadata(entity);
        return this.registry.set(entity, metadata).get(entity);
    }
}
exports.CollectionMetadata = CollectionMetadata;
//# sourceMappingURL=metadata.js.map